// Eine fertige Kurve aus cos-Termen

#include <math.h>

#include "kurve.h"

#include "ppm.h"

// t in [0,1]
static double kurve_eval1 (double ** d, double t, double phi)
{
    double w = *(*d)++;
    size_t n = 0.1 + *(*d)++;
    
    for (size_t k = 1; k <= n; k++)
    {
        double a = *(*d)++;
        double f = *(*d)++;
        w += a * cos (t*k + f + phi);
    }
    
    return w;
}

// Auswertung mit Phasenverschiebung
p_t kurve_eval_phi (double * d, double t, p_t phi)
{
    t *= *d++;
    double x = kurve_eval1 (&d, t, phi.x);
    double y = kurve_eval1 (&d, t, phi.y);
    
    return (p_t) {x, y};
}

p_t kurve_eval (double * d, double t)
{
    return kurve_eval_phi (d, t, (p_t) {0,0});
}

// (1-l)*a(t) + l*b(t)
p_t kurve_eval_mix (double * a, double * b, double t, double l, p_t phi_a, p_t phi_b)
{
    p_t pa = kurve_eval_phi (a, t, phi_a);
    p_t pb = kurve_eval_phi (b, t, phi_b);
    
    return (p_t) {(1-l)*pa.x + l*pb.x, (1-l)*pa.y + l*pb.y};
}

// phi = Drehung um x/y-Achse
void kurve_plot (double * ku, int color0, int color1, double step, p_t phi)
{
    double t_max = *ku / (2*M_PI);
    for (double t=0.0; t < 1.0; t += step)
        dot (kurve_eval_phi (ku, t, phi), t < t_max ? color0 : color1, 1);
}

// Animationen

void kurve_ani_rotate (double * ku, const char * name, size_t k1, int color0, int color1)
{
    double fak = 2.0*M_PI / k1;
    for (size_t k = 1; k <= k1; k++)
    {
        ppm_clear();
        kurve_plot (ku, color0, color1, 0.01, (p_t) { fak * (k-1), 0 });
        ppm_write (name, k);
    }
}

void kurve_ani_morph (double * a, double * b, const char * name, size_t frames, size_t pix)
{
    p_t phi_a = {0,0};
    p_t phi_b = {0,0};
    
    size_t k, f, f2 = frames/2;
    
    for (f = 0; f < f2; f++)
    {
        double l = (double) f / f2;
        for (k=0; k < pix; k++)
        {
            double t = (double) k / pix;
            dot (kurve_eval_mix (a, b, t, l, phi_a, phi_b), CL_BLACK, 1);
        }
        ppm_write (name, f);
        ppm_clear();
    }

    for (f = 0; f < f2; f++)
    {
        double l = (double) f / f2;

        phi_a.y = M_PI*l + M_PI;
        phi_b.y = M_PI*l;
        for (k=0; k < pix; k++)
        {
            double t = (double) k / pix;
            dot (kurve_eval_mix (a, b, t, 1-l, phi_a, phi_b), CL_BLACK, 1);
        }
        ppm_write (name, f+f2);
        ppm_clear();
    }
}
