#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "lambda.h"

// lambda ist eine streng monoton wachsende Abbildung
// [0..n] -> [0,1]
lambda_t * lambda_alloc (unsigned int n)
{
    lambda_t * l = (lambda_t*) calloc (1, sizeof (lambda_t));
    
    l->n = n;
    l->t = (double*) calloc (n+1, sizeof (double));
    
    for (unsigned int k=0; k <= n; k++)
        l->t[k] = (double) k/n;
    
    return l;
}

void lambda_free (lambda_t * l)
{
    free (l->t);
    free (l);
}

// kopiert l0 nach l. 
// l und l0 mssen allokiert worden sein und den gleichen
// Urbildbereich haben
void lambda_copy (lambda_t * l, const lambda_t * l0)
{
    l->n = l0->n;
    memcpy (l->t, l0->t, (l0->n+1) * sizeof(double));
}

static double eps = 1e-4;

// Verschiebt ab dem k-ten Wert j Werte um maximal delta.
// Liefert 1 bei Erfolg bzw. gltigem Ergebnis
int lambda_change (lambda_t * l, const lambda_t * l0, 
                   unsigned int k, unsigned int j, double delta)
{
    unsigned int i;
    if (k <= 0 || k+j-1 >= l->n || j < 1)
        return 0;
    
    lambda_copy (l, l0);
    
    double * t = l->t;
    
    double ta = t[k-1];
    double tb = t[k+j];
    
    for (i=k; i < k+j; i++)
        t[i] += delta * sin (M_PI * (t[i]-ta) / (tb-ta));
    
    for (i=0; i < l->n; i++)
        if (t[i+1] < t[i]+eps)
            return 0;
    
    return 1;
}

// l so anpassen, da die Abstnde der Zielpunkte gleich sind
void lambda_adjust_len (lambda_t * l, double * x, double * y, double sigma)
{
    double len[l->n+1];
    len[0] = 0.0;
    for (size_t k=1; k <= l->n; k++)
        len[k] = len[k-1] + hypot (sigma, hypot (x[k % l->n] - x[k-1],
                                                 y[k % l->n] - y[k-1]));
    for (size_t k=0; k < l->n; k++)
        l->t[k] = len[k] / len[l->n];
}

