// http://www.gnu.org/software/gsl/manual/html_node/Fast-Fourier-Transforms.html

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "trafo.h"
#include "lambda.h"
#include "spline.h"
#include "tupel.h"

#include "error.h"
#include "ppm.h"
#include "kurve.h"

// Koordinaten den Originaldaten auf dem
// Einheitskreis. 0 -> (1,0) und wir durchlaufen
// mathematisch positiv

int fno;

// Die Anzahl der Punkte fr die FFT der Interpolierten
#define M 64

// den k-ten FFT-Koeffizient bewerten
static double bwert (int k, double w)
{
    w = pow (k, 1) * pow (0.01+w, 0.9);

    return w;
}

// die FFT bewerten
static double bewerte_fft (trafo_t * f)
{
    p_t ck;
    double wert = 0.0;
    
    for (unsigned int k=0; k <= f->n - k; k++)
    {
        // Addiere die Amplitudenquadrate
        // http://de.wikipedia.org/wiki/Additionstheorem#Sinusoid
        ck = to_a_phi (f->cx, k, f->n);  wert += bwert (k, ck.x);
        ck = to_a_phi (f->cy, k, f->n);  wert += bwert (k, ck.x);
    }
        
    return wert;
}


static void plot_xy (trafo_t * f, lambda_t * l)
{
//return;
    trafo_plot_xy (f, l, CL_PINK, CL_BLUE);
    ppm_write ("bild-%d.ppm", ++fno);
    ppm_clear();
}

int main (void)
{
    //trafo_plot_h();
    //return 0;
    //////////////////////////////////////////////////////////////////////
    // Punktmenge, die anzunhern ist
    
    //tupel_t * v = tupels[TUPEL_KREIS];
    //tupel_t * v = tupels[TUPEL_A];
    tupel_t * v = tupels[TUPEL_HERZ];
    
    if (1)
    {
        tupel_plot_num (v, CL_BLACK, 1);
        tupel_plot (v, CL_RED, 4);
        ppm_write ("bild-%d.ppm", ++fno);
        ppm_clear();
    }
    
    fft_alloc (M);
    trafo_t * f = trafo_alloc (M);
    
    lambda_t * l = lambda_alloc (v->dim);
    spline_t * s = spline_alloc (v->dim, INTERPOL_SPLINE);
    
    //////////////////////////////////////////////////////////////////////
    // Spline damit initialisieren und zeichnen
    spline_init (s, l, v->x, v->y);
    
    // Grafik in Datei ausgeben
    // Die N roten Vorgabepunkte ber alles drber zeichnen
    spline_plot (s, CL_BLUE, 4*PIX);
    tupel_plot (v, CL_RED, 4);
    
    ppm_write ("bild-%d.ppm", ++fno);
    ppm_clear();
    
    //////////////////////////////////////////////////////////////////////
    // Die Zeit-Koordinaten der Punkte so verschieben, 
    // da die Zeitabstnde den Abstnden im Raum entsprechen
    
    if (1)
    {
        lambda_adjust_len (l, v->x, v->y, 0.0 / l->n);
        spline_init (s, l, v->x, v->y);
        
        // Grafik in Datei ausgeben
        spline_plot (s, CL_BLUE, 4*PIX);
        tupel_plot (v, CL_RED, 4);
        
        ppm_write ("bild-%d.ppm", ++fno);
        ppm_clear();
        
    }
    
    //////////////////////////////////////////////////////////////////////
    // FFT fr diesen Spline bestimmen
    trafo_fft (f, s);

    plot_xy (f, l);
    
    // nicht-optimierte FFT zeichnen 
    trafo_plot (f, CL_GREEN, CL_YELLOW, 0.01, l->t[l->n-1]);
    tupel_plot (v, CL_RED, 4);
    ppm_write ("bild-%d.ppm", ++fno);
    ppm_clear();
    
    //////////////////////////////////////////////////////////////////////
    // FFT-Koeffizienten anhand der obigen Bewertung optimieren
    
    printf ("\n=== Bewertung ===\n");
    printf (" W_start = % .8f\n", bewerte_fft (f));
    
    trafo_fft (f, s);
    trafo_crop (f, 0.06);
    trafo_plot (f, CL_GREEN, CL_YELLOW, 0.01, l->t[l->n-1]);
    tupel_plot (v, CL_RED, 4);
    ppm_write ("bild-%d.ppm", ++fno);
    ppm_clear();
    
    trafo_fft (f, s);
    
    int change = trafo_optimize (f, l, v->x, v->y, trafo_fft, bewerte_fft);
    
    plot_xy (f, l);
    
    printf (" Changes= %d\n", change);
    printf (" W_opt   = % .8f\n", bewerte_fft (f));
    
    // Den optimierten FFT-Grafen zeichnen
    trafo_plot (f, CL_GREEN, CL_YELLOW, 0.01, l->t[l->n-1]);
    tupel_plot (v, CL_RED, 4);
    
    ppm_write ("bild-%d.ppm", ++fno);
    ppm_clear();
    
   
    //////////////////////////////////////////////////////////////////////
    // Kleine FFT-Koeffizienten wegwerfen
    
    int num = trafo_crop (f, 0.06);
    
    plot_xy (f, l);
    
    trafo_dump ("\n=== F{%s(t)} ===\n", f);
    
    printf (" W_crop  = % .8f,\t # Koeff = %d\n", bewerte_fft (f), num);
    
    // Den grnen FFT-Grafen zeichnen
    trafo_plot (f, CL_GREEN, CL_YELLOW, 0.01, l->t[l->n-1]);
    tupel_plot (v, CL_RED, 4);
        
    ppm_write ("bild-%d.ppm", ++fno);
    ppm_clear();
    
    // Den schwarzen End FFT-Grafen zeichnen
    trafo_plot (f, CL_BLACK, v->id == TUPEL_HERZ ? CL_BLACK : CL_NIL, 0.01, l->t[l->n-1]);
        
    ppm_write ("bild-%d.ppm", ++fno);
    ppm_clear();
    
    //////////////////////////////////////////////////////////////////////
    
    FILE * data_a = fopen ("data_avr.c", "w");
    trafo_out_kurve_C_AVR (data_a, f, "data", l->t[l->n-1]);
    FILE * data_c = fopen ("data.c", "w");
    trafo_out_kurve_C (data_c, f, "data", l->t[l->n-1]);

    //////////////////////////////////////////////////////////////////////
    // Aufrumen
    fft_free();
    spline_free (s);
    lambda_free (l);
    
    return 0;
}


