#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

// http://de.wikipedia.org/wiki/Portable_Pixmap
#include "error.h"
#include "ppm.h"

//////////////////////////////////////////////////////////////////////
// Text zeichnen
//////////////////////////////////////////////////////////////////////
#include "vektor-zeichen.h"

static void font_xy (vf_uint_t x, vf_uint_t y)
{
    plot (x, y, CL_BLACK);
}
    
void font_skip (void)
{
}

vecfont_t font = 
{
    .beam_xy   = font_xy,
    .beam_skip = font_skip,
    .zoom = 1
};

//////////////////////////////////////////////////////////////////////
// Grafik-Zeugs
//////////////////////////////////////////////////////////////////////

// Palette fr PPM-Grafik
#define COLOR_MAX 15

static const char * const COLORS[] = 
{
    [CL_WHITE] = " 15 15 15 ",
    [CL_BLACK] = "  0  0  0 ",
    [CL_GREEN] = "  0 12  0 ",
    [CL_LIME]  = "  3 15  3 ",
    [CL_CYAN]  = "  0 13 13 ",
    [CL_BLUE]  = "  4  4 15 ",
    [CL_ROSE]  = " 15 12 11 ",
    [CL_PINK]  = " 13  0 15 ",
    [CL_BROWN] = " 10  4  0 ",
    [CL_ORANGE]= " 15 12  4 ",
    [CL_YELLOW]= " 14 14  0 ",
    [CL_RED]   = " 15  0  0 "
};

// Pixel-Array fr PPM-Grafik
static uint8_t ppm[PIX][PIX];

// Zum Zeichnen von Kringeln
static const int DOT_XY[][12] =
{
    { 0 },
    { 1, 0, -1, 0 },
    { 2, 2, 1, 0, -1, -1, 0, 1 },
    { 2, 2, 2, 1, 0, -1, -2, -2, -2, -1, 0, 1 }
};

int x2p (double x)
{
    return PIX/2 * (1.0 + x / SCALE);
}

double p2x (int p)
{
    return 2.0*p*SCALE / PIX - SCALE;
}

void plot (int x, int y, unsigned int color)
{
    if (x >= 0 && x < PIX 
        && y >= 0 && y < PIX  
        && color < CL_NIL)
    {    
        ppm[x][y] = color;
    }
}

// Zeichnet einen Punkt mit Offset (x,y)
static void dot1 (p_t p, int color, int x, int y)
{
    plot (x + x2p (p.x), y + x2p (p.y), color);
}

// Zeichnet einen Punkt/Kringel mit Radius r
void dot (p_t p, int color, int r)
{
    if (color >= CL_NIL)
        return;
    if (r-- == 1)
        return dot1 (p, color, 0, 0);
        
    if (r > 3) r = 3;    
    
    for (int n=0; n < 4*r; n++)
    {
        int dx = DOT_XY[r][n];
        int dy = DOT_XY[r][(n+r) % (4*r)];
        dot1 (p, color, dx, dy);
    }
}

void ppm_text (p_t p, const char * text, int zoom)
{
    font.text = text;
    font.start.x = x2p (p.x);
    font.start.y = x2p (p.y);
    font.zoom = zoom;
    
    while (! vecfont_draw_letters (&font));
}

void ppm_clear (void)
{
    memset (ppm, 0, sizeof (ppm));
}

// Schreibt PIX x PIX PPM-Grafik aus ppm[][]
void out_ppm (FILE * stream)
{
    fprintf (stream, "P3\n%d %d\n%d\n", PIX, PIX, COLOR_MAX);
    
    for (int y = 0; y < PIX; y++)
    {
        for (int x = 0; x < PIX; x++)
            fprintf (stream, COLORS[ppm[x][PIX-1-y] % CL_NIL]);
            
        fprintf (stream, "\n");
    }
}

void ppm_write (const char * format, ...)
{
    static char fname[100];
    
    va_list args;
    va_start (args, format);
    
    vsprintf (fname, format, args);
    
    va_end (args);
    
    FILE * file = fopen (fname, "w");

    if (NULL == file)
        error ("Kann Bild nicht nach '%s' schreiben!", fname);
    
    out_ppm (file);
    fclose (file);
}
