#ifndef TRAFO_H
#define TRAFO_H

#include <stdio.h>

#include "point.h"
#include "lambda.h"
#include "spline.h"

#define REAL(A,B)    (A[2*(B)])
#define IMAG(A,B)    (A[2*(B)+1])

// Das Ergebnis der FFT
typedef struct
{
    unsigned int n;
    // Koefizzienten von x(t), y(t) in C (je N Stck)
    double *cx, *cy;
    // Fr gepackte FFT-Koordinaten
    double *x, *y;
    
    // Anzahl verbleibender cos-Terme nach Vernachlssigung k
    // einer Amplituden
    unsigned int m[2];
    //double *c[2];
    
    // Gradient
    double *grad;
    size_t grad_dim, num_p;
    double *v;
} trafo_t;

extern trafo_t * trafo_alloc (unsigned int n);
extern p_t to_cos_sin (double * c, unsigned int k, unsigned int n);
extern p_t to_a_phi (double * c, unsigned int k, unsigned int n);

extern p_t trafo_eval (trafo_t * f, double t);
extern p_t trafo_eval_aphi (trafo_t * f, double t, double phi);
extern p_t trafo_eval_dt (trafo_t * f, double t);
extern int trafo_crop (trafo_t * f, double thres);

extern void trafo_init_xy (trafo_t * f, spline_t * s);
extern void trafo_fft_xy (trafo_t * f);
extern void trafo_fft (trafo_t * f, spline_t * s);
extern void trafo_dwt (trafo_t * f, spline_t * s);
extern void trafo_dwt_xy (trafo_t * f);
extern void trafo_dwt_invert (trafo_t * f);
extern void trafo_plot_dwt (trafo_t * f, int color0, int color1, double max_t);
extern void trafo_plot_y (trafo_t * f, int color);
extern void trafo_plot_cy (trafo_t * f, int color);

extern int trafo_optimize (trafo_t*, lambda_t*, double*, double*, void(*)(trafo_t*, spline_t*), double(*)(trafo_t*));

extern void trafo_out_kurve_C (FILE * file, trafo_t * f, const char *name, double t_max);
extern void trafo_out_kurve_C_AVR (FILE * file, trafo_t * f, const char *name, double t_max);
extern void trafo_dump (const char *text, trafo_t * f);

extern void trafo_plot (trafo_t * f, int color0, int color1, double step, double max_t);
extern void trafo_plot_xy (trafo_t * f, lambda_t * l, int colorx, int colory);

extern void fft_alloc (size_t n);
extern void fft_free (void);

#include <gsl/gsl_fft_real.h>
#include <gsl/gsl_fft_halfcomplex.h>

extern gsl_fft_real_wavetable * real;
extern gsl_fft_real_workspace * work;

#endif // TRAFO_H
