#include <math.h>

#include "tupel.h"

static p_t HERZ_PIXEL[] = 
{
    {0.0,  1.0},
    
    {0.0,  1.3},
    {1.6,  2.5},
    {3.0,  1.0},
    {2.0, -1.0},
    {0.0, -3.2},
    
    {0.0, -3.5},
    
    {-0.0, -3.2},
    {-2.0, -1.0},
    {-3.0,  1.0},
    {-1.6,  2.5},
    {-0.0,  1.3}
};

tupel_t tupel_herz = 
{
    .id = TUPEL_HERZ,
    .dim = sizeof (HERZ_PIXEL) / sizeof (HERZ_PIXEL[0]),
    .lo_left  = { -3.0, -4.0}, 
    .up_right = {  3.0,  2.5},
    .inv_y = 0,
    .p = HERZ_PIXEL
};

static p_t A_PIXEL[] = 
{
    { 51, 201 },
    { 45, 190 },
    { 58, 183 },
    { 67, 200 },
    { 73, 219 },
    { 83, 237 },
    { 96, 247 },
    { 96, 228 },
    { 96, 197 },
    { 96, 185 },
    { 96, 179 },
    { 96, 185 },
    { 91, 202 },
    { 78, 219 },
    { 62, 210 },
    { 67, 200 },
    {105, 201 }
};

tupel_t tupel_A = 
{
    .id = TUPEL_A,
    .dim = sizeof (A_PIXEL) / sizeof (A_PIXEL[0]),
    .lo_left  = { 41, 176}, 
    .up_right = {106, 250},
    .inv_y = 0,
    .p = A_PIXEL
};

#define VV 30
#define WW (-20)
static p_t B_PIXEL[] = 
{
    { 339-WW, 241 },
    { 287-WW, 263 },
    { 267-WW, 444 },
    { 261-WW, 603 },
    { 211-WW, 656 }, 
    { 240,    600 }, 
    { 398-VV, 662 },
    { 521-VV, 541 },
    { 395-VV, 388 },
    { 397-VV, 415 },
    { 495-VV, 258 },
    { 370-VV, 145 },
    { 226-WW, 188 },
    { 249-WW, 237 }
};

tupel_t tupel_B = 
{
    .id = TUPEL_B,
    .dim = sizeof (B_PIXEL) / sizeof (B_PIXEL[0]),
    .lo_left  = {176,     144}, 
    .up_right = {176+348, 144+522},
    .inv_y = 1,
    .p = B_PIXEL
};

static double KREIS_T[] =
{
    0, 0.1, 0.3, 0.4,
    0.6, 0.9, 0.95
};

tupel_t tupel_kreis = 
{
    .id = TUPEL_KREIS,
    .dim = sizeof (KREIS_T) / sizeof (KREIS_T[0]),
    .lo_left  = { -1, -1},
    .up_right = {  1,  1},
    .inv_y = 0
};

tupel_t * tupels[] = 
{
    [TUPEL_A] =     & tupel_A,
    [TUPEL_B] =     & tupel_B,
    [TUPEL_HERZ]  = & tupel_herz,
    [TUPEL_KREIS] = & tupel_kreis
};

void tupel_plot (tupel_t * v, int color, int diam)
{
    for (size_t k=0; k < v->dim; k++)
        dot (v->p[k], color, diam);
}

void tupel_plot_num (tupel_t * v, int color, int zoom)
{
    for (size_t k=1; k <= v->dim; k++)
    {
        static char text[10];
        sprintf (text, "%d", k);
        p_t p = v->p[k-1];
        
        p.x += 4 * DPIX;
        p.y += 4 * DPIX;
        
        switch (v->id)
        {
            case TUPEL_A:
                if (4 == k || 10 == k)
                    continue;
                if (16 == k) { sprintf (text, "%d=%d", k,4); p.x -= 18*DPIX; }
                if (12 == k) { sprintf (text, "%d=%d", k,10); p.x -= 18*DPIX; }
                break;
            case TUPEL_HERZ:
                if (2 == k || 6 == k)
                    continue;
                if (1 == k || 7 == k)
                    p.y -= 12 * DPIX;
                if (8  == k) { sprintf (text, "%d=%d", k,6); p.x -= 11*DPIX; }
                if (12 == k) { sprintf (text, "%d=%d", k,2); p.x -= 18*DPIX; }
                break;
        }
        
        ppm_text (p, text, zoom);
    }
}

void __attribute__((constructor))
init_tupel (void)
{
    size_t k, j;
    tupel_t * v = & tupel_kreis;
    
    v->p = (p_t*) calloc (v->dim, sizeof (p_t));
    
    for (k = 0; k < v->dim; k++)
        v->p[k] = (p_t)
        {
            cos (2.0 * M_PI * KREIS_T[k]),
            sin (2.0 * M_PI * KREIS_T[k])
        };
        
    for (j = 0; j < sizeof (tupels) / sizeof (tupels[0]); j++)
    {
        v = tupels[j];
        double x0 = v->lo_left.x; double x1 = v->up_right.x;
        double y0 = v->lo_left.y; double y1 = v->up_right.y;
        
        v->x = (double*) calloc (v->dim, sizeof (double));
        v->y = (double*) calloc (v->dim, sizeof (double));
        for (k = 0; k < v->dim; k++)
        {
            // skaliere auf [-1,1]^2
            v->x[k] = 2.0 * (v->p[k].x - x0) / (x1-x0) -1.0;
            v->y[k] = 2.0 * (v->p[k].y - y0) / (y1-y0) -1.0;
            if (v->inv_y)
                v->y[k] = - v->y[k];
                
            v->p[k].x = v->x[k];
            v->p[k].y = v->y[k];
        }
    }
}

void __attribute__((destructor))
fini_tupel (void)
{
    free (tupel_kreis.p);
    
    for (size_t j = 0; j < sizeof (tupels) / sizeof (tupels[0]); j++)
    {
        free (tupels[j]->x);
        free (tupels[j]->y);
    }
}
